<?php
namespace Teknisa\DF\Services;

use Teknisa\Libs\Util\Environment;
use Zeedhi\Framework\DB\StoredProcedure\Param;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;

class ConfereErrosAvisosNota {

    /**
     * Informa que determinada ocorrencia (erro ou aviso) foi identificado e precisa ser notificado.
     */
    const ID_ERRO_HABILITADO   = 1;

    /**
     * Informa que determinada ocorrencia nao precisa ser notificada.
     */
    const ID_ERRO_DESABILITADO = 0;

    private $avisosNF = array(
        'AV'   => 'Nota Fiscal com avisos.',
        'AV12' => 'Data de Entrada da nota fiscal difere da Data Programada na AF.',
        'AV01' => 'Fornecedor da nota fiscal difere do fornecedor da AF.',
        'AV15' => 'Data de vencimento da nota fiscal difere da data programada na AF.',
        'AV17' => 'Data de Vencimento informada anterior a data de processamento.'
    );

    private $errosNF = array(
        'ER'   => 'Nota Fiscal com erros.',
        'ER05' => 'Soma das entradas sem nota fiscal difere do valor total da nota.',
        'ER02' => 'Total da nota fiscal difere da soma dos itens.',
        'ER03' => 'Total de vencimentos difere valor total nota fiscal. '
    );

    private $avisosItemproduto = array(
        'AV09' => 'Item com código da situação tributária inválido.',
        'AV10' => 'Item com código fiscal de operação inválido.',
        'AV11' => 'Quantidade de produto informada difere da quantidade prevista na AF.',
        'AV13' => 'Valor unitário do produto informado difere do valor unitário previsto na AF.',
        'AV16' => 'Item informado na nota fiscal difere do produto programado na AF.',
        'AV'   => 'Produtos da nota fiscal com avisos.'
    );

    private $errosItemproduto = array(
        'ER'   => 'Produtos da nota fiscal com erros.',
        'ER04' => 'Item com valor unitário igual a zero.',
        'ER'   => 'Produtos da nota fiscal com erros.',
        'ER03' => 'Total de vencimentos difere valor total nota fiscal.'
    );

    private $avisosImposnftot = array(
        'AV07' => 'Valor de Imposto informado diferente da tabela.',
        'AV08' => 'Imposto da NF difere da alíquota do fornecedor.'
    );

    private $errosImposnftot = array(
        'ER01' => 'Imposto da NF inválido.',
        'ER05' => 'Soma das entradas sem nota fiscal difere do valor total da nota.',
        'ER02' => 'Total nota fiscal difere soma dos itens.',
        'ER03' => 'Total de vencimentos difere valor total nota fiscal.'
    );

    private $avisosItemproimpos = array(
        'AV02' => 'Percentual de imposto da tabela difere do percentual informado',
        'AV04' => 'Imposto do item da entrada com código fiscal inválido.',
        'AV05' => 'Imposto do item com situação tributária(CST) inválida.',
        'AV06' => 'Imposto do item com cód. fiscal operação - CFOp inválido.',
        'AV03' => 'Percentual Imposto do item difere do valor Imposto do Item.'
    );

    private $semLocal = array(
        'AV14' => 'Quantidade de produto informada difere da quantidade recebida.',
        'ER06' => 'Soma das saídas sem nota fiscal difere do valor total da nota.'
    );

    private $entityManager;
    private $filial;
    private $lancamento;
    private $notafiscal;
    private $errosAvisos = array();

	public function __construct($entityManager, $filial, $lancamento) {
        $this->entityManager = $entityManager;
        $this->setNotaFiscalEntity($filial, $lancamento);
	}

	public function setNotaFiscalEntity($filial, $lancamento) {
	    $this->filial = $filial;
        $this->lancamento = $lancamento;

        $this->notafiscal = $this->entityManager->getRepository('Teknisa\Model\Fiscal\Entities\Notafiscal')
            ->findOneBy(array('nrlanctonf' => $lancamento, 'cdfilial' => $filial));
	}

    public function executaProcedureConfereErrosNf() {
        try {
            $valorLancamento = $this->notafiscal->getVrnotafisc();

            $this->callProcedureConfereErros($valorLancamento);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    private function callProcedureConfereErros($valorLancamento) {
        try {
            $procedure = new StoredProcedure($this->entityManager->getConnection(), 'CONFERE_ERROS_NF');
            $procedure->addParam(new Param('P_CDFILIAL',     Param::PARAM_INPUT, $this->filial,     Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_NRLANCTONF',   Param::PARAM_INPUT, $this->lancamento, Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_VRLANCAMENTO', Param::PARAM_INPUT, $valorLancamento,  Param::PARAM_TYPE_STR));
            $procedure->execute();
        } catch (\Exception $e) {
            throw $e;
        }
    }

    private function addErrosAvisos(array $errosAvisos, $index, $item = null, $imposto = null) {
        $keys = array_keys($errosAvisos);
        $errosAvisosKey = $keys[$index];
        $message = $errosAvisos[$errosAvisosKey];
        $this->errosAvisos[$errosAvisosKey] = array();
        $this->errosAvisos[$errosAvisosKey]["DSERROAVISO"] = $message;
        $this->errosAvisos[$errosAvisosKey]["ITEMERRO"]    = $item;
        $this->errosAvisos[$errosAvisosKey]["IMPOSTOERRO"] = $imposto;
    }

    private function validaErrosAvisos($idErroAviso, array $errosAvisos, $produto = null, $imposto = null) {
        $impostoErro = null;
        $itemErro    = null;
        for ($i = 0; $i < strlen($idErroAviso); $i++) {
            if ($idErroAviso[$i] == self::ID_ERRO_HABILITADO) {
                if($imposto) {
                    $dadosErro = $this->getItemImpostoErro($imposto->getCdfilial(), $imposto->getNrlanctonf(), $imposto->getNrseqitprod(), $imposto->getCdimposto() );
                    $itemErro = $dadosErro["CDPRODUTO"] . ' - ' . $dadosErro["NMPRODUTO"];
                    $impostoErro = $dadosErro["CDIMPOSTO"] . ' - ' . $dadosErro["NMIMPOSTO"];
                }
                else if($produto) {
                    $dadosErro = $this->getItemErro($produto->getCdfilial(), $produto->getNrlanctonf(), $produto->getNrseqitprod());
                    $itemErro = $dadosErro["CDPRODUTO"] . ' - ' . $dadosErro["NMPRODUTO"];
                }
                $this->addErrosAvisos($errosAvisos, $i, $itemErro, $impostoErro);
            }
        }
    }

    public function getErrosAvisos($cdFilial) {
        $errosAvisos = array();

        if(!empty($this->errosAvisos)) {
            
            $sql = '(';
            foreach($this->errosAvisos as $key => $erro) {
                $sql.= "'".$key."'".',';
            }
            $sql = substr($sql,0, strlen($sql)-1).')';
            $erros = $this->getErroAvisoNf($sql, $cdFilial);
            foreach($erros as $erro) {
                $erro["DSERROAVISO"] = $this->errosAvisos[$erro["CDERROAVISO"]]["DSERROAVISO"];
                $erro["ITEMERRO"]    = $this->errosAvisos[$erro["CDERROAVISO"]]["ITEMERRO"];
                $erro["IMPOSTOERRO"]    = $this->errosAvisos[$erro["CDERROAVISO"]]["IMPOSTOERRO"];
                $errosAvisos[] = $erro;
            }
   
        }
        return $errosAvisos;
    }

    public function confereErrosNotaFiscal() {
        $this->entityManager->refresh($this->notafiscal);

        $idavisos = $this->notafiscal->getIdavisos();
        $iderros  = $this->notafiscal->getIderros();

        for ($i = 1; $i < strlen($idavisos) && $i < count($this->avisosNF); $i++) {
            if ($idavisos[$i] == self::ID_ERRO_HABILITADO) {
                $this->addErrosAvisos($this->avisosNF, $i);
            }
        }

        for ($i = 1; $i < strlen($iderros) && $i < count($this->errosNF); $i++) {
            if ($iderros[$i] == self::ID_ERRO_HABILITADO) {
                $this->addErrosAvisos($this->errosNF, $i);
            }
        }
    }

    public function confereErrosItemprodutos() {
        $avisos = array();
        $filial = $this->notafiscal->getCdfilial();
        $lancamento = $this->notafiscal->getNrlanctonf();


        $itens = $this->entityManager->getRepository('Teknisa\Model\Fiscal\Entities\Itemproduto')
            ->findBy(array('cdfilial' => $filial, 'nrlanctonf' => $lancamento));
        
        foreach ($itens as $item) {
            $idavisos = $item->getIdavisositem();
            $iderros  = $item->getIderrosItem();
            
            $this->validaErrosAvisos($idavisos, $this->avisosItemproduto, $item, null);
            $this->validaErrosAvisos($iderros, $this->errosItemproduto, $item, null);
        }
        return $avisos;
    }

    public function confereErrosImposnftot() {
        $errosAvisos = array();
        $filial = $this->notafiscal->getCdfilial();
        $lancamento = $this->notafiscal->getNrlanctonf();

        //Trabimposnftot -> Imposnftot
        $impostos = $this->entityManager->getRepository('Teknisa\Model\Fiscal\Entities\Imposnftot')
            ->findBy(array('cdfilial' => $filial, 'nrlanctonf' => $lancamento));

        foreach ($impostos as $imposto) {
            $idavisostot = $imposto->getIdavisimptt();
            $iderrostot  = $imposto->getIderrosimptt();
            $this->validaErrosAvisos($idavisostot, $this->avisosImposnftot);
            $this->validaErrosAvisos($iderrostot, $this->errosImposnftot);
        }
        return $errosAvisos;
    }

    public function confereErrosItemproimpos() {
        $errosAvisos = array();
        $filial = $this->notafiscal->getCdfilial();
        $lancamento = $this->notafiscal->getNrlanctonf();

        // Trabitproimpos -> Itemproimpos
        $impostos = $this->entityManager->getRepository('Teknisa\Model\Fiscal\Entities\Itemproimpos')
            ->findBy(array('cdfilial' => $filial, 'nrlanctonf' => $lancamento));

        foreach ($impostos as $imposto) {
            $idavisimpit = $imposto->getIdavisimpit();
            $this->validaErrosAvisos($idavisimpit, $this->avisosItemproimpos, null, $imposto);
        }

        return $errosAvisos;
    }

    public function confereErros($cdFilial) {
        $this->confereErrosNotaFiscal();
        $this->confereErrosItemprodutos();
        $this->confereErrosImposnftot();
        $this->confereErrosItemproimpos();

        return $this->getErrosAvisos($cdFilial);
    }
    
    public function getErroAvisoNf($cdErros, $cdFilial) {
        $sql = "SELECT CDERROAVISO,
                  IDERROAVISO
                FROM ERROAVISONF
                WHERE CDERROAVISO   IN $cdErros
                AND CDERROAVISO NOT IN
                  (SELECT CDERROAVISOFIL FROM ERROAVISONFFIL WHERE CDFILIAL = :P_CDFILIAL
                  )
                UNION
                SELECT CDERROAVISOFIL AS CDERROAVISO,
                  IDERROAVISOFIL      AS IDERROAVISO
                FROM ERROAVISONFFIL
                WHERE CDERROAVISOFIL IN $cdErros
                AND CDFILIAL          = :P_CDFILIAL";
                
        $params = array("P_CDFILIAL" => $cdFilial);
        
        $std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute($params);
		return $std->fetchAll(); 
    }
    
    public function getErrosItem() {
        $sql = "SELECT IDAVISOSITEM, IDERROSITEM
            FROM ITEMPRODUTO
            WHERE CDFILIAL  = :P_CDFILIAL
            AND NRLANCTONF  = :P_NRLANCTONF";
                
        $params = array("P_CDFILIAL" => $this->filial,
                        "P_NRLANCTONF" => $this->lancamento);
        
        $std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute($params);
		return $std->fetchAll(); 
    }
    
    public function getItemErro($cdfilial, $nrlanctonf, $nrseqitem) {
        $sql = "SELECT
                    P.CDPRODUTO,
                    P.NMPRODUTO
                FROM
                    ITEMPRODUTO   IP,
                    PRODUTO       P
                WHERE
                    IP.NRSEQITPROD = :P_NRSEQITPROD
                    AND P.CDPRODUTO = IP.CDPRODUTO
                    AND IP.CDFILIAL = :P_CDFILIAL
                    AND IP.NRLANCTONF = :P_NRLANCTONF";
                    
        $params = array("P_CDFILIAL"    => $cdfilial,
                        "P_NRLANCTONF"  => $nrlanctonf, 
                        "P_NRSEQITPROD" => $nrseqitem);
            
        $std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute($params);
		return $std->fetch(); 
    }
    
    public function getItemImpostoErro($cdfilial, $nrlanctonf, $nrseqitem, $cdimposto) {
        $sql = "SELECT
                P.CDPRODUTO,
                P.NMPRODUTO,
                I.CDIMPOSTO,
                I.NMIMPOSTO
            FROM ITEMPRODUTO   IP,
                PRODUTO       P,
                IMPOSTO       I
            WHERE I.CDIMPOSTO = :P_CDIMPOSTO
            AND IP.NRSEQITPROD = :P_NRSEQITPROD
            AND P.CDPRODUTO = IP.CDPRODUTO
            AND IP.CDFILIAL = :P_CDFILIAL
            AND IP.NRLANCTONF = :P_NRLANCTONF"; 
            
        $params = array("P_CDFILIAL"    => $cdfilial,
                        "P_NRLANCTONF"  => $nrlanctonf,
                        "P_NRSEQITPROD" => $nrseqitem, 
                        "P_CDIMPOSTO"   => $cdimposto);
            
        $std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute($params);
		return $std->fetch(); 
    }
}